/*
Module : EZView.cpp
Purpose: Implements the CWinApp class for EZView

Copyright (c) 2000 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise)
when your product is released in binary form. You are allowed to modify the source code in any way you want
except you cannot modify the copyright details at the top of each module. If you want to distribute source
code with your application, then you are only allowed to distribute versions released by the author. This is
to maintain a single distribution point for the source code.

*/


////////////////////////// Includes ///////////////////////////////////////////

#include "stdafx.h"
#include "EZView.h"
#include "MainFrm.h"
#include "EZViewDoc.h"
#include "EZViewView.h"
#include "OptionsPropertySheet.h"


////////////////////////// Macros / Defines ///////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG

CEZViewApp theApp;


////////////////////////// Implementation /////////////////////////////////////

CMainFrame* GetMainFrame()
{
  return static_cast<CMainFrame*>(AfxGetMainWnd());
}


CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


BEGIN_MESSAGE_MAP(CEZViewApp, CWinAppEx)
  ON_COMMAND(ID_APP_ABOUT, &CEZViewApp::OnAppAbout)
  ON_COMMAND(ID_FILE_OPEN, &CEZViewApp::OnFileOpen)
  ON_COMMAND(ID_FILE_OPEN_FOLDER, &CEZViewApp::OnFileOpenFolder)
  ON_UPDATE_COMMAND_UI(ID_FILE_MRU_FOLDER1, &CEZViewApp::OnUpdateRecentFolderMenu)
  ON_COMMAND_EX_RANGE(ID_FILE_MRU_FOLDER1, ID_FILE_MRU_FOLDER16, &CEZViewApp::OnOpenRecentFolder)
  ON_UPDATE_COMMAND_UI(ID_FILE_MRU_FILE1, &CEZViewApp::OnUpdateRecentFileMenu)
  ON_COMMAND_EX_RANGE(ID_FILE_MRU_FILE1, ID_FILE_MRU_FILE16, &CEZViewApp::OnOpenRecentFile)
  ON_COMMAND(ID_TOOLS_OPTIONS, &CEZViewApp::OnToolsOptions)
  ON_COMMAND(ID_FILE_NEW, &CWinAppEx::OnFileNew)
  ON_COMMAND(ID_FILE_OPEN, &CWinAppEx::OnFileOpen)
  ON_COMMAND(ID_HELP_INDEX, &CEZViewApp::OnHelpIndex)
  ON_COMMAND(ID_HELP, &CEZViewApp::OnHelp)
END_MESSAGE_MAP()

CEZViewApp::CEZViewApp() : m_RecentFolderList(0, _T("Recent Directory List"), _T("Directory%d"), 4),
                           m_RecentFileList(0, _T("Recent File List"), _T("File%d"), 4),
                           m_nCurrentImageIndex(-1),
                           m_bPromptStartEndFolder(false),
                           m_bHideCursorDuringFullScreen(false),
                           m_bMessageBoxUp(false),
                           m_nSlideshowInterval(0),
                           m_bShowFilenameDuringFullScreen(false),
                           m_FullScreenTextColor(RGB(0, 255, 0)),
                           m_FullScreenBackgroundColor(RGB(0, 0, 0)),
                           m_bAskMeOnDelete(false)
{
  SetAppID(_T("Naughter.EZView.1.0"));
}

BOOL CEZViewApp::InitInstance()
{
  //Initialize common control
  INITCOMMONCONTROLSEX InitCtrls;
  InitCtrls.dwSize = sizeof(InitCtrls);
  InitCtrls.dwICC = ICC_WIN95_CLASSES;
  if (!InitCommonControlsEx(&InitCtrls))
    return FALSE;

  //Let the base class do its thing
  if (!__super::InitInstance())
    return FALSE;

  //seed the random number generator
  srand(static_cast<unsigned int>(time(nullptr)));

  SetRegistryKey(_T("PJ Naughter"));

  //Load up the user settings
  LoadUserSettings();
  m_RecentFolderList.ReadList();
  m_RecentFileList.ReadList();

  EnableTaskbarInteraction(TRUE);
  InitContextMenuManager();
  InitTooltipManager();
  CMFCToolTipInfo ttParams;
  ttParams.m_bVislManagerTheme = TRUE;
  GetTooltipManager()->SetTooltipParams(AFX_TOOLTIP_TYPE_ALL, RUNTIME_CLASS(CMFCToolTipCtrl), &ttParams);

  //Register document templates
  CSingleDocTemplate* pDocTemplate;
  pDocTemplate = new CSingleDocTemplate(
    IDR_MAINFRAME,
    RUNTIME_CLASS(CEZViewDoc),
    RUNTIME_CLASS(CMainFrame),
    RUNTIME_CLASS(CEZViewView));
  AddDocTemplate(pDocTemplate);

  //Parse command line for standard shell commands, DDE, file open
  CCommandLineInfo cmdInfo;
  ParseCommandLine(cmdInfo);

  //Dispatch commands specified on the command line
  if (!ProcessShellCommand(cmdInfo))
    return FALSE;
  m_pMainWnd->ShowWindow(SW_SHOW);
  m_pMainWnd->UpdateWindow();
  m_pMainWnd->DragAcceptFiles();

  return TRUE;
}

void CEZViewApp::OnAppAbout()
{
  CAboutDlg aboutDlg;
  aboutDlg.DoModal();
}

void CEZViewApp::OnFileOpen() 
{
  CString sExt;
  if (!sExt.LoadString(IDS_FILE_EXTENSIONS))
    return;
  CFileDialog dlg(TRUE, nullptr, nullptr, OFN_HIDEREADONLY | OFN_FILEMUSTEXIST, sExt);
  if (dlg.DoModal() == IDOK)
  {
    CString sFile = dlg.GetPathName();
    CDocument* pDoc = OpenDocumentFile(sFile);
    if (pDoc != nullptr)
      AddToRecentFileList(sFile);
  }
}

bool CEZViewApp::OpenDocumentFolder(_In_ const CString& sFolder)
{
  //what will be the return value from this method (assume the worst)
  bool bSuccess = false;

  //Get a listing of the directory
  RefreshImageList(sFolder);

  //Display the first file in the listing if there is some files found
  if (m_ImagesInCurrentFolder.size())
  {
    CDocument* pDoc = OpenDocumentFile(m_ImagesInCurrentFolder[0]);
    if (pDoc != nullptr)
    {
      bSuccess = true;
      AddToRecentFolderList(sFolder);
    }
  }
  else
    AfxMessageBox(IDS_FOLDER_DOES_NOT_CONTAIN_ANY_IMAGES);

  return bSuccess;
}

void CEZViewApp::OnFileOpenFolder() 
{
  //Browse for a directory
  CString sTitle;
  if (!sTitle.LoadString(IDS_FOLDER_OPEN_TITLE))
    return;
  CString sDir;
  if (GetShellManager()->BrowseForFolder(sDir, AfxGetMainWnd(), nullptr, sTitle, BIF_RETURNONLYFSDIRS, nullptr))
    OpenDocumentFolder(sDir);
}

void CEZViewApp::RefreshImageList2(_In_ const CString& sPath)
{
  TCHAR szPath[_MAX_PATH];
  szPath[0] = _T('\0');
  TCHAR szDrive[_MAX_DRIVE];
  szDrive[0] = _T('\0');
  TCHAR szDir[_MAX_DIR];
  szDir[0] = _T('\0');
  _tsplitpath_s(sPath, szDrive, sizeof(szDrive)/sizeof(TCHAR), szDir, sizeof(szDir)/sizeof(TCHAR), nullptr, 0, nullptr, 0);
  _tmakepath_s(szPath, sizeof(szPath)/sizeof(TCHAR), szDrive, szDir, nullptr, nullptr);
  RefreshImageList(szPath);
}

class CSortFilenames
{
public:
  inline bool operator()(const CString& sFile1, const CString& sFile2)
  {
    return sFile1.CompareNoCase(sFile2) < 0;
  }
};

void CEZViewApp::RefreshImageList(_In_ const CString& sFolder)
{
  //Validate our parameters
  ASSERT(sFolder.GetLength());

  m_nCurrentImageIndex = -1;

  //Form the search path
  CString sSearchPath;
  if (sFolder[sFolder.GetLength() - 1] == _T('\\'))
    sSearchPath = sFolder + _T("*.*");
  else
    sSearchPath = sFolder + _T("\\*.*");

  //Find the first file in this directory which we can display
  m_ImagesInCurrentFolder.clear();
  CFileFind find;
  BOOL bFind = find.FindFile(sSearchPath);  
  while (bFind)
  {
    bFind = find.FindNextFile();
    CString sFilename(find.GetFilePath());
    if (CanDisplayFile(sFilename))
      m_ImagesInCurrentFolder.push_back(sFilename);
  }

  //Sort the array
  std::sort(m_ImagesInCurrentFolder.begin(), m_ImagesInCurrentFolder.end(), CSortFilenames());

  TRACE(_T("Found %d images in folder %s\n"), static_cast<int>(m_ImagesInCurrentFolder.size()), sFolder.GetString());
}

bool CEZViewApp::CanDisplayFile(_In_ const CString& sFilename)
{
  //Base whether we can display the file on the extension
  //of its filename. This is not the most ideal but is the
  //best we can do
  TCHAR szExt[_MAX_EXT];
  szExt[0] = _T('\0');
  _tsplitpath_s(sFilename, nullptr, 0, nullptr, 0, nullptr, 0, szExt, sizeof(szExt)/sizeof(TCHAR));
  return ((_tcsicmp(szExt, _T(".jpg")) == 0) ||  
          (_tcsicmp(szExt, _T(".jpeg")) == 0) ||
          (_tcsicmp(szExt, _T(".bmp")) == 0) ||
          (_tcsicmp(szExt, _T(".dib")) == 0) ||
          (_tcsicmp(szExt, _T(".wmf")) == 0) ||
          (_tcsicmp(szExt, _T(".ico")) == 0) ||
          (_tcsicmp(szExt, _T(".gif")) == 0) ||
          (_tcsicmp(szExt, _T(".tif")) == 0) ||
          (_tcsicmp(szExt, _T(".tiff")) == 0) ||
          (_tcsicmp(szExt, _T(".png")) == 0) ||
          (_tcsicmp(szExt, _T(".exif")) == 0) ||
          (_tcsicmp(szExt, _T(".emf")) == 0));
}

CDocument* CEZViewApp::OpenDocumentFile(LPCTSTR lpszFileName) 
{
  //Let the base class do its thing
  CDocument* pDoc = __super::OpenDocumentFile(lpszFileName);

  //Set the image index back to unknown
  m_nCurrentImageIndex = -1;
  if (pDoc != nullptr)
    pDoc->UpdateAllViews(nullptr);

  return pDoc;
}

CDocument* CEZViewApp::OpenNextDocumentFile()
{
  int nImagesInCurrentFolder = static_cast<int>(m_ImagesInCurrentFolder.size());
  for (int i=m_nCurrentImageIndex+1; i<nImagesInCurrentFolder; i++)
  { 
    CDocument* pDoc = theApp.OpenDocumentFile(i);
    if (pDoc != nullptr)
      return pDoc;
  }

  return nullptr;
}

CDocument* CEZViewApp::OpenPrevDocumentFile()
{
  for (int i=m_nCurrentImageIndex-1; i>=0; i--)
  {
    CDocument* pDoc = OpenDocumentFile(i);
    if (pDoc != nullptr)
      return pDoc;
  }

  return nullptr;
}

CDocument* CEZViewApp::OpenDocumentFile(_In_ int nImageIndex)
{
  CDocument* pDoc = theApp.OpenDocumentFile(m_ImagesInCurrentFolder[nImageIndex]);
  if (pDoc != nullptr)
  {
    m_nCurrentImageIndex = nImageIndex;
    pDoc->UpdateAllViews(nullptr);
  }

  return pDoc;
}

BOOL CEZViewApp::OnOpenRecentFolder(UINT nID) 
{
  //Validate our parameters
  ASSERT(nID >= ID_FILE_MRU_FOLDER1);
  ASSERT(nID < (ID_FILE_MRU_FOLDER1 + static_cast<UINT>(m_RecentFolderList.GetSize())));
  
  int nIndex = nID - ID_FILE_MRU_FOLDER1;
  ASSERT(m_RecentFolderList[nIndex].GetLength() != 0);

  TRACE(_T("MRU: open folder (%d) '%s'.\n"), nIndex + 1, m_RecentFolderList[nIndex].GetString());

  if (!OpenDocumentFolder(m_RecentFolderList[nIndex]))
    m_RecentFolderList.Remove(nIndex);

  return TRUE;
}

void CEZViewApp::OnUpdateRecentFolderMenu(CCmdUI* pCmdUI)
{
  m_RecentFolderList.UpdateMenu(pCmdUI);
}

BOOL CEZViewApp::OnOpenRecentFile(UINT nID) 
{
  ASSERT(nID >= ID_FILE_MRU_FILE1);
  ASSERT(nID < ID_FILE_MRU_FILE1 + static_cast<UINT>(m_RecentFileList.GetSize()));

  int nIndex = nID - ID_FILE_MRU_FILE1;
  ASSERT((m_RecentFileList)[nIndex].GetLength() != 0);

  TRACE(_T("MRU: open file (%d) '%s'.\n"), nIndex + 1, m_RecentFileList[nIndex].GetString());

  if (OpenDocumentFile((m_RecentFileList)[nIndex]) == nullptr)
    m_RecentFileList.Remove(nIndex);

  return TRUE;
}

void CEZViewApp::OnUpdateRecentFileMenu(CCmdUI* pCmdUI)
{
  m_RecentFileList.UpdateMenu(pCmdUI);
}

void CEZViewApp::AddToRecentFolderList(_In_z_ LPCTSTR lpszPathName)
{
  //Validate our parameters
  ASSERT(lpszPathName != nullptr);

  m_RecentFolderList.Add(lpszPathName);
}

void CEZViewApp::AddToRecentFileList(_In_z_ LPCTSTR lpszPathName)
{
  //Validate our parameters
  ASSERT(lpszPathName != nullptr);

  m_RecentFileList.Add(lpszPathName);
}

int CEZViewApp::ExitInstance() 
{
  //Save the Folder and File MRU
  m_RecentFolderList.WriteList();
  m_RecentFileList.WriteList();
  
  //Let the parent class do its thing  
  return __super::ExitInstance();
}

void CEZViewApp::OnHelpIndex()
{
  WinHelp(0, HELP_INDEX);
}

void CEZViewApp::OnHelp()
{
  WinHelp(0, HELP_INDEX);
}

void CEZViewApp::WinHelp(DWORD /*dwData*/, UINT /*nCmd*/) 
{
  //Get the exe name and add a extension of ".htm" and just ShellExecute it!!
  TCHAR szPath[_MAX_PATH];
  szPath[0] = _T('\0');
  if (GetModuleFileName(AfxGetInstanceHandle(), szPath, _MAX_PATH) == 0)
    return;
  TCHAR szDrive[_MAX_DRIVE];
  szDrive[0] = _T('\0');
  TCHAR szDir[_MAX_DIR];
  szDir[0] = _T('\0');
  TCHAR szFname[_MAX_FNAME];
  szFname[0] = _T('\0');
  _tsplitpath_s(szPath, szDrive, sizeof(szDrive)/sizeof(TCHAR), szDir, sizeof(szDir)/sizeof(TCHAR), szFname, sizeof(szFname)/sizeof(TCHAR), nullptr, 0);
  _tmakepath_s(szPath, sizeof(szPath)/sizeof(TCHAR), szDrive, szDir, szFname, _T("htm"));

  HINSTANCE hRun = ShellExecute(m_pMainWnd ? m_pMainWnd->GetSafeHwnd() : nullptr, _T("open"), szPath, nullptr, nullptr, SW_HIDE); 
  if (hRun <= reinterpret_cast<HINSTANCE>(32))
  {
    CString sMsg;
    AfxFormatString1(sMsg, IDS_FAIL_VIEW_HELP, szPath);
    AfxMessageBox(sMsg);
  }
}


CMyRecentFileList::CMyRecentFileList(UINT nStart, LPCTSTR lpszSection, LPCTSTR lpszEntryFormat, int nSize, int nMaxDispLen) : CRecentFileList(nStart, lpszSection, lpszEntryFormat, nSize, nMaxDispLen)
{
}

void CMyRecentFileList::UpdateMenu(CCmdUI* pCmdUI)
{
  ENSURE_ARG(pCmdUI != nullptr);
  AFXASSUME(m_arrNames != nullptr);

  CMenu* pMenu = pCmdUI->m_pMenu;
  if (pCmdUI->m_pSubMenu != nullptr)
    pMenu = pCmdUI->m_pSubMenu;

  if (m_strOriginal.IsEmpty() && pMenu != nullptr)
    pMenu->GetMenuString(pCmdUI->m_nID, m_strOriginal, MF_BYCOMMAND);

  if (m_arrNames[0].IsEmpty())
  {
    // no MRU files
    if (!m_strOriginal.IsEmpty())
      pCmdUI->SetText(m_strOriginal);
    pCmdUI->Enable(FALSE);
    return;
  }

  if (pMenu == nullptr)
    return;

  for (int iMRU = 0; iMRU < m_nSize; iMRU++)
    pMenu->DeleteMenu(pCmdUI->m_nID + iMRU, MF_BYCOMMAND);

  TCHAR szCurDir[_MAX_PATH];
  DWORD dwDirLen = GetCurrentDirectory(_MAX_PATH, szCurDir);
  if (dwDirLen == 0 || dwDirLen >= _MAX_PATH)
    return;	// Path too long

  int nCurDir = static_cast<int>(_tcslen(szCurDir));
  ASSERT(nCurDir >= 0);
  szCurDir[nCurDir] = '\\';
  szCurDir[++nCurDir] = '\0';

  CString strName;
  CString strTemp;
  for (int iMRU = 0; iMRU < m_nSize; iMRU++)
  {
    if (!GetDisplayName(strName, iMRU, szCurDir, nCurDir))
      break;

    // double up any '&' characters so they are not underlined
    LPCTSTR lpszSrc = strName;
    LPTSTR lpszDest = strTemp.GetBuffer(strName.GetLength() * 2);
    while (*lpszSrc != 0)
    {
      if (*lpszSrc == '&')
        *lpszDest++ = '&';
      if (_istlead(*lpszSrc))
        *lpszDest++ = *lpszSrc++;
      *lpszDest++ = *lpszSrc++;
    }
    *lpszDest = 0;
    strTemp.ReleaseBuffer();

    // insert mnemonic + the file name
    TCHAR buf[10];
    int nItem = ((iMRU + m_nStart) % _AFX_MRU_MAX_COUNT) + 1;


    // number &1 thru &9, then 1&0, then 11 thru ...
    if (nItem > 10)
      _stprintf_s(buf, _countof(buf), _T("%d "), nItem);
    else if (nItem == 10)
      Checked::tcscpy_s(buf, _countof(buf), _T("1&0 "));
    else
      _stprintf_s(buf, _countof(buf), _T("&%d "), nItem);

    pMenu->InsertMenu(pCmdUI->m_nIndex++,
      MF_STRING | MF_BYPOSITION, pCmdUI->m_nID++,
      CString(buf) + strTemp);
  }

  // update end menu count
  pCmdUI->m_nIndex--; // point to last menu added
  pCmdUI->m_nIndexMax = pMenu->GetMenuItemCount();

  pCmdUI->m_bEnableChanged = TRUE;    // all the added items are enabled
}

void CEZViewApp::LoadUserSettings()
{
  m_bPromptStartEndFolder         = GetProfileInt(_T("General"), _T("PromptStartEndDirectory"), true) != 0;
  m_bAskMeOnDelete                = GetProfileInt(_T("General"), _T("AskOnDelete"), true) != 0;
  m_nSlideshowInterval            = GetProfileInt(_T("Slideshow"), _T("Interval"), 5);
  m_FullScreenTextColor           = static_cast<COLORREF>(GetProfileInt(_T("FullScreen"), _T("TextColor"), RGB(0, 255, 0)));
  m_FullScreenBackgroundColor     = static_cast<COLORREF>(GetProfileInt(_T("FullScreen"), _T("BackgroundColor"), RGB(0, 0, 0)));
  m_bHideCursorDuringFullScreen   = GetProfileInt(_T("FullScreen"), _T("HideCursor"), false) != 0;
  m_bShowFilenameDuringFullScreen = GetProfileInt(_T("FullScreen"), _T("ShowFilename"), false) != 0;
}

void CEZViewApp::SaveUserSettings()
{
  WriteProfileInt(_T("General"), _T("PromptStartEndDirectory"), m_bPromptStartEndFolder);
  WriteProfileInt(_T("General"), _T("AskOnDelete"), m_bAskMeOnDelete);
  WriteProfileInt(_T("Slideshow"), _T("Interval"), m_nSlideshowInterval);
  WriteProfileInt(_T("FullScreen"), _T("TextColor"), m_FullScreenTextColor);
  WriteProfileInt(_T("FullScreen"), _T("BackgroundColor"), m_FullScreenBackgroundColor);
  WriteProfileInt(_T("FullScreen"), _T("HideCursor"), m_bHideCursorDuringFullScreen);
  WriteProfileInt(_T("FullScreen"), _T("ShowFilename"), m_bShowFilenameDuringFullScreen);
}

void CEZViewApp::OnToolsOptions() 
{
  CString sCaption;
  if (!sCaption.LoadString(IDS_OPTIONS))
    return;
  COptionsPropertySheet sheet(sCaption);
  sheet.m_Page1.m_nSlideshowInterval            = m_nSlideshowInterval;
  sheet.m_Page1.m_bPromptStartEndFolder         = m_bPromptStartEndFolder;
  sheet.m_Page1.m_TextColor                     = m_FullScreenTextColor;
  sheet.m_Page1.m_BackgroundColor               = m_FullScreenBackgroundColor;
  sheet.m_Page1.m_bHideCursorDuringFullScreen   = m_bHideCursorDuringFullScreen;
  sheet.m_Page1.m_bShowFilenameDuringFullScreen = m_bShowFilenameDuringFullScreen;
  sheet.m_Page1.m_bAskMeOnDelete                = m_bAskMeOnDelete;
  sheet.m_psh.dwFlags |= PSH_NOAPPLYNOW;
  if (sheet.DoModal() == IDOK)	
  {
    m_nSlideshowInterval            = sheet.m_Page1.m_nSlideshowInterval;
    m_bPromptStartEndFolder         = sheet.m_Page1.m_bPromptStartEndFolder == TRUE;
    m_FullScreenTextColor           = sheet.m_Page1.m_TextColor;
    m_FullScreenBackgroundColor     = sheet.m_Page1.m_BackgroundColor;
    m_bHideCursorDuringFullScreen   = sheet.m_Page1.m_bHideCursorDuringFullScreen == TRUE;
    m_bShowFilenameDuringFullScreen = sheet.m_Page1.m_bShowFilenameDuringFullScreen == TRUE;
    m_bAskMeOnDelete                = sheet.m_Page1.m_bAskMeOnDelete == TRUE;
    SaveUserSettings();
  }
}

int CEZViewApp::DoMessageBox(LPCTSTR lpszPrompt, UINT nType, UINT nIDPrompt)
{
  m_bMessageBoxUp = true;
  int nReturn = __super::DoMessageBox(lpszPrompt, nType, nIDPrompt);
  m_bMessageBoxUp = false;
  return nReturn;
}
